function FrtDemo(action)

%FrtDemo(signal)
%
% Fractional Fourier Transform and 
% Filtering in Fractional Domains Demonstrations
%
% If no input signal is given, Function rect is used


persistent x sig figNumber hndlList;

if nargin<1,
    action='initialize';
    N = 128;
    x = zeros(1,N);
    x(3*N/8:5*N/8) = 4/N;
    sig = [];
elseif nargin==1
  if ~ischar(action)
       x = action(:).';
       % If necessery is padded to even length (duplicating last sample)
       if rem(length(x),2)
          x = [ x x(end) ];
       end
       sig = x;
       action = 'initialize';
   end
end


if strcmp(action,'initialize'),
   %====================================
   % Information for all buttons
    labelColor=[0.9 0.9 0.9];

   
    figNumber=figure( ...
        'Name','Fractional Fourier Transform & Filtering Demonstration', ...
        'NumberTitle','off', ...
        'Backingstore','off', ...
        'Visible','off');
    
    %==================================
    % Set up the axes
    axHndl1=axes( ...
        'Units','normalized', ...
        'Position',[0.10 0.60 0.60 0.32], ...
        'Drawmode','fast', ...
        'Visible','on');
    axHndl2=axes( ...
        'Units','normalized', ...
        'Position',[0.10 0.2 0.60 0.32], ...
        'Drawmode','fast', ...
        'Visible','on');


    %=================================
    % Set up the scroll bar
    sbBottom=0.1;
    sbLeft=0.05;
    barHeight = 0.04; 
    barWidth = .675;
    textWidth = barWidth/4;
    scrollPos = [sbLeft sbBottom barWidth barHeight];
    callbackStr = 'FrtDemo(''slide'')';
    slideHndl = uicontrol( ...
         'Style','slider', ...
         'Units','normalized', ...
         'BackgroundColor',labelColor,...
         'Position',scrollPos, ...
         'Value',0, ...
         'userdata',0, ...
         'min',-1, ...
         'max',1, ...
         'SliderStep',[0.001 0.1],...
         'Callback',callbackStr);
 
    % Scrollbar title
    rangePos = [sbLeft+textWidth sbBottom-barHeight 2*textWidth barHeight];
    centerHndl = uicontrol( ...
	'Style','text', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
	'Position',rangePos, ...
	'Horiz','center', ...
	'String','Order of FRT');

    % Left and right range indicators
    rangePos = [sbLeft sbBottom-barHeight textWidth barHeight];
    leftHndl = uicontrol( ...
	'Style','text', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
	'Position',rangePos, ...
	'Horiz','left', ...
	'String','-1');

    rangePos = [sbLeft+3*textWidth sbBottom-barHeight textWidth barHeight];
    rightHndl = uicontrol( ...
	'Style','text', ...
       'Units','normalized', ...
         'BackgroundColor',labelColor,...
	'Position',rangePos, ...
	'Horiz','right', ...
	'String','1');

    %====================================
    % Information for all buttons (and menus)
    labelColor=[0.9 0.9 0.9];
    yInitPos=0.90;
    menutop=0.95;
    top=0.4;
    left=0.785;
    btnWid=0.175;
    btnHt=0.06;
    textHeight = 0.05;
    textWidth = 0.07;

    % Spacing between the button and the next command's label
    spacing=0.025;
    
    %====================================
    % The CONSOLE frame
    frmBorder=0.019; frmBottom=0.04; 
    frmHeight = 0.92; frmWidth = btnWid;
    yPos=frmBottom-frmBorder;
    frmPos=[left-frmBorder yPos frmWidth+2*frmBorder frmHeight+2*frmBorder];
    h=uicontrol( ...
        'Style','frame', ...
        'Units','normalized', ...
        'Position',frmPos, ...
	'BackgroundColor',[0.5 0.5 0.5]);

    %====================================
    % The SIGNAL CHOICE Menu
    menuNumber=1;
    yPos=menutop-(menuNumber-1)*(btnHt+spacing);

    if ~isempty(sig)
       labelStr='Rect|Chirp|4 Chirps|Chirp Triangle|Signal';
    else
       labelStr='Rect|Chirp|4 Chirps|Chirp Triangle';
    end  
    callbackStr='FrtDemo(''signal'')';

    % Generic button information
    btnPos=[left yPos-btnHt btnWid btnHt];
    signalHndl=uicontrol( ...
        'Style','popupmenu', ...
        'Units','normalized', ...
         'BackgroundColor',labelColor,...
        'Position',btnPos, ...
        'String',labelStr, ...
        'Callback',callbackStr);

    if ~isempty(sig)
         set(signalHndl,'Value',5)
    end

    %====================================
    % SUB-CONSOLE frame for sweep related
    subfrmBorder=0.009; subfrmBottom=0.55; 
    subfrmHeight = 0.32; subfrmWidth = btnWid;
    yPos=subfrmBottom-subfrmBorder;
    subfrmPos=[left-subfrmBorder yPos subfrmWidth+2*subfrmBorder subfrmHeight+2*subfrmBorder];
    h=uicontrol( ...
        'Style','frame', ...
        'Units','normalized', ...
        'Position',subfrmPos, ...
	'BackgroundColor',[0.6 0.6 0.6]);

    %====================================
    % Menu : View 3D Sweeping of area
     menuNumber=2;
     yPos=menutop-(menuNumber-1)*(btnHt+spacing);

    labelStr='Sweep Area';
    callbackStr='FrtDemo(''view3d'');';

    % Generic button information
    btnPos=[left yPos-btnHt btnWid btnHt];
    btn1Hndl=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...
        'Position',btnPos, ...
        'String',labelStr, ...
 	'BackgroundColor',labelColor, ...
       'Callback',callbackStr);


    %===================================
    % 3D View Limits

    %===================================
    % 1. Alpha start
    top = yPos - btnHt - spacing;
    labelWidth = frmWidth-textWidth-.01;
    labelBottom=top-textHeight;
    labelLeft = left;
    labelRight = left+btnWid;
    labelPos = [labelLeft labelBottom labelWidth textHeight];
    h = uicontrol( ...
	'Style','text', ...
        'Units','normalized', ...
	'Position',labelPos, ...
        'Horiz','left', ...
        'String','start:', ...
        'Interruptible','off', ...
	'BackgroundColor',[0.5 0.5 0.5], ...
        'ForegroundColor','white');

    % Text field
    textPos = [labelRight-textWidth labelBottom textWidth textHeight];
    callbackStr = 'FrtDemo(''start'')';
    startHndl = uicontrol( ...
	'Style','edit', ...
        'Units','normalized', ...
	'Position',textPos, ...
	'Horiz','right', ...
	'Background','white', ...
        'Foreground','black', ...
	'String','0','Userdata',0, ...
        'callback',callbackStr);

    %===================================
    % 2. Alpha end
    top = yPos - 2*btnHt - 2*spacing;
    labelWidth = frmWidth-textWidth-.01;
    labelBottom=top-textHeight;
    labelLeft = left;
    labelRight = left+btnWid;
    labelPos = [labelLeft labelBottom labelWidth textHeight];
    h = uicontrol( ...
	'Style','text', ...
        'Units','normalized', ...
	'Position',labelPos, ...
        'Horiz','left', ...
        'String','end:', ...
        'Interruptible','off', ...
	'BackgroundColor',[0.5 0.5 0.5], ...
        'ForegroundColor','white');

    % Text field
    textPos = [labelRight-textWidth labelBottom textWidth textHeight];
    callbackStr = 'FrtDemo(''aend'')';
    endHndl = uicontrol( ...
	'Style','edit', ...
        'Units','normalized', ...
	'Position',textPos, ...
	'Horiz','right', ...
	'Background','white', ...
        'Foreground','black', ...
	'String','1','Userdata',1, ...
        'callback',callbackStr);

    %===================================
    % 3. Alpha step
    top = yPos - 3*btnHt - 3*spacing;
    labelWidth = frmWidth-textWidth-.01;
    labelBottom=top-textHeight;
    labelLeft = left;
    labelRight = left+btnWid;
    labelPos = [labelLeft labelBottom labelWidth textHeight];
    
    h = uicontrol( ...
	'Style','text', ...
        'Units','normalized', ...
	'Position',labelPos, ...
        'Horiz','left', ...
        'String','Step:', ...
        'Interruptible','off', ...
	'BackgroundColor',[0.5 0.5 0.5], ...
        'ForegroundColor','white');

    % Text field
    textPos = [labelRight-textWidth labelBottom textWidth textHeight];
    callbackStr = 'FrtDemo(''step'')';
    stepHndl = uicontrol( ...
	'Style','edit', ...
        'Units','normalized', ...
	'Position',textPos, ...
	'Horiz','right', ...
	'Background','white', ...
        'Foreground','black', ...
	'String','0.1','Userdata',.1, ...
        'callback',callbackStr);

    %===========================================
    % SUB-CONSOLE frame for Filter related Btn
    subfrmBorder=0.009; subfrmBottom=0.35; 
    subfrmHeight = 0.155; subfrmWidth = btnWid;
    yPos=subfrmBottom-subfrmBorder;
    subfrmPos=[left-subfrmBorder yPos subfrmWidth+2*subfrmBorder subfrmHeight+2*subfrmBorder];
    h=uicontrol( ...
        'Style','frame', ...
        'Units','normalized', ...
        'Position',subfrmPos, ...
	'BackgroundColor',[0.6 0.6 0.6]);

%top = labelBottom-4*spacing;
top = .5;
    %====================================
    % button 1  The FILTER Button
    % 
     btnNumber=1;
     yPos=top-(btnNumber-1)*(btnHt+spacing);
    
    labelStr='Filter';
    callbackStr='FrtDemo(''filter'');';

    % Generic button information
    btnPos=[left yPos-btnHt btnWid btnHt];
    filterHndl=uicontrol( ...
       'Style','push', ...
       'Units','normalized', ...
       'BackgroundColor',labelColor,...
       'Position',btnPos, ...
       'String',labelStr, ...
       'Callback',callbackStr);

    %====================================
    % button 2
    btnNumber=2;
    yPos=top-(btnNumber-1)*(btnHt+spacing);
    labelStr='View spectrog';

    % Generic button information
    btnPos=[left yPos-btnHt btnWid btnHt];
    spectroHndl=uicontrol( ...
        'Style','checkbox', ...
        'Units','normalized', ...
        'Position',btnPos, ...
  	'BackgroundColor',labelColor, ...
        'String',labelStr, ...
        'Value',0);

    %====================================
    % button 2
    btnNumber=3;
    yPos=top-.025-(btnNumber-1)*(btnHt+spacing);
    labelStr='View Phase';
    callbackStr='FrtDemo(''slide'');';
    
    % Generic button information
    btnPos=[left yPos-btnHt btnWid btnHt];
    phaseHndl=uicontrol( ...
        'Style','checkbox', ...
        'Units','normalized', ...
        'Position',btnPos, ...
 	'BackgroundColor',labelColor, ...
        'String',labelStr, ...
        'Value',0,...
        'Callback',callbackStr);

    %====================================
    % button 2
%    btnNumber=2;
%    yPos=top-(btnNumber-1)*(btnHt+spacing);
%    labelStr='3-D plot';
%    callbackStr='sosdemo(''viewtoggle'');';
%
%    % Generic button information
%    btnPos=[left yPos-btnHt btnWid btnHt];
%    btn2Hndl=uicontrol( ...
%        'Style','checkbox', ...
%        'Units','normalized', ...
%        'Position',btnPos, ...
%        'String',labelStr, ...
%        'Value',1, ...
%        'Callback',callbackStr);


    %====================================
    % The INFO button
    labelStr='Info';
    callbackStr='FrtDemo(''info'')';
    helpHndl=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...
 	'BackgroundColor',labelColor, ...
        'Position',[left frmBottom+btnHt+spacing btnWid btnHt], ...
        'String',labelStr, ...
        'Callback',callbackStr);

    %====================================
    % The CLOSE button
    labelStr='Close';
    callbackStr='close(gcf)';
    closeHndl=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...
 	'BackgroundColor',labelColor, ...
        'Position',[left frmBottom btnWid btnHt], ...
        'String',labelStr, ...
        'Callback',callbackStr);


    hndlList=[ axHndl1 axHndl2 slideHndl signalHndl ...
               startHndl endHndl stepHndl btn1Hndl spectroHndl phaseHndl ];
    
    set(figNumber, ...
	'Visible','on', ...
	'UserData',hndlList);

    FrtDemo('slide')

%====================================
% 		Actions
%====================================



elseif strcmp(action,'slide'),
   hndlList = get(figNumber,'UserData');
   alpha = get(hndlList(3),'value');
   if alpha == 0
       frt_x = x;
   else
       frt_x = fracft(x,alpha);
   end
   figure(figNumber)
   axes(hndlList(1))
      plot(abs(frt_x))
      title( sprintf('Fractional Fourier Transform of order : %.3g ', alpha))
   axes(hndlList(2))
      if get(hndlList(10),'Value')			% Spectrogram or Phase
         hold off
         plot(unwrap(angle(frt_x)))
         title('Unwraped Phase')
      else
         N = length(x);
         hold off
         nfft = max(32,2^nextpow2(N/32));
         [imtf,axf,axt] = specgram(x,nfft,[],blackman(nfft),max(31,floor(3*nfft/4)));
         imagesc(axt,axf,20*log10(abs(imtf)+eps)),   axis xy, colormap(jet)
          
         hold on
         if alpha >=0
            plot([ 0 cos(alpha*pi/2)*axt(end)/10 ], ...
                 [0 sin(alpha*pi/2)*axf(end)/5],'w')
         else
            plot([ 0 cos(alpha*pi/2)*axt(end)/10 ], ...
                 [axf(end) axf(end)+sin(alpha*pi/2)*axf(end)/5],'w')
         end
         title('Signal spectrogram and fractional path ')
      end

elseif strcmp(action,'filter'),
   
   hndlList = get(figNumber,'UserData');
   alpha = get(hndlList(3),'value');
   if alpha == 0
       frt_x = x;
   else
       frt_x = fracft(x,alpha);
   end
   
   N = length(frt_x);
   
   Fact_Norm = 4.8105;   
   h=figure;
       plot(abs(frt_x))
       title('Left Click 2 points to delimit the Segment you want to keep')
       
   [xc,yc] = ginput(2);
   xc = sort(xc);
	
   xc1 = floor(xc(1));
   xc2 = ceil(xc(2));
	
   % On ne conserve que la zone (on annulle le reste)
   frtxfilt = frt_x;
   if ((xc1~=0)|(xc2~=1))
      frtxfilt(1:xc1) = 0;
   end
   if (xc2~=N)
      frtxfilt(xc2:N) = 0;
   end
   
   ondsynt = (fracft(frtxfilt,-alpha));          % Here ondsynt might be complex
   ondsynt = Fact_Norm * ondsynt;

   Fa = sqrt(N);
   ta = (0:N-1)/Fa;

   figure(h), clf
   subplot(311)
      hold on; plot(ta,abs(frt_x),'b')
      plot(ta,abs(frtxfilt),'r')
      title([ '\alpha = ' num2str(alpha) ' FRT modulus' ])
      legend('Filtered part','signal FRT') 
      grid on

   if get(hndlList(9),'Value')			% Spectrogram or Time Waveform
       subplot(312)
         nfft = max(32,2^nextpow2(N/32));
         specgram(real(ondsynt),nfft,[],blackman(nfft),max(31,floor(3*nfft/4)))
         title('Synthesized filtered signal')
      subplot(313)	
         specgram(real(x),nfft,[],blackman(nfft),max(31,floor(3*nfft/4)))
         title('Original Signal')
  
   else 
      subplot(312)
         plot(ta,real(ondsynt),'r')
         title('Synthesized filtered signal')
         grid on
      subplot(313)	
         plot(ta,real(x),'b')
         title('Original Signal')
         grid on
   end


elseif strcmp(action,'start'),
    v = get(gco,'String');
    v = eval(v);
    set(gco,'Userdata',v,'String',num2str(v))

elseif strcmp(action,'aend'),
    v = get(gco,'String');
    v = eval(v);
    set(gco,'Userdata',v,'String',num2str(v))

elseif strcmp(action,'step'),
    v = get(gco,'String');
    v = eval(v);
%    s = get(gco,'String');
%   vv = eval(s,num2str(v));
%    if vv<1, vv = v; end
%    vv = round(vv);
%    set(gco,'Userdata',vv,'String',num2str(vv))
    set(gco,'Userdata',v,'String',num2str(v))


elseif strcmp(action,'view3d'),
    hndlList = get(figNumber,'Userdata');
    
    deb = get(hndlList(5),'Userdata');
    aend = get(hndlList(6),'Userdata');
    step = get(hndlList(7),'Userdata');
 
    x = [ x(:) ; zeros(1,rem(length(x),2)) ];
    N = length(x);


    fe = sqrt(N);
    t=(-N/2:N/2-1)*1/fe;

    alpha = deb:step:aend;

    x_alpha = zeros(N,length(alpha));

    for k = 1:length(alpha)
        frtx=fracft(x,alpha(k));
        x_alpha(:,k) = (frtx).';
    end

   % Modulus
     figure
        colormap(cool)
        mesh(alpha,t,abs((x_alpha)));
        view(70,50)
        title( sprintf('FRT Modulus for \alpha \in [%.2g;%.2g]',deb,aend) )
        xlabel('\alpha')
        ylabel('t_\alpha')
        set(gca,'Ylim',[t(1) t(N)])
        grid on	
	
    % Real Part
%	figure
%	colormap(gray)
%	caxis([5 20])
%	mesh(alpha,t,real((x_alpha)));
%	view(70,50)
%	title('Partie relle')
%	xlabel('\alpha')
%	ylabel('t_\alpha')
%	set(gca,'Ylim',[t(1) t(N)])
%	grid on
 
    
        
elseif strcmp(action,'signal'),
    hndlList = get(figNumber,'Userdata');
    
    nb = get(hndlList(4),'Value');
      
    if nb == 1      % Rect
        N = 128;
        x = zeros(1,N);
        x(3*N/8:5*N/8) = 4/N;
    
    elseif nb == 2                     % Linear Chirp
        t = 0.001:0.001:2;                 % 2 secs @ 1kHz sample rate
        x = chirp(t,0,1,150);          % Start @ DC, cross 150Hz at t=1sec 

    elseif nb == 3
        t = 0.001:0.001:2;                 % 2 secs @ 1kHz sample rate
        x = chirp(t,0,1,150);          % Start @ DC, cross 150Hz at t=1sec 
        x = x+chirp(t,100,1,250);                                          
        x = x+chirp(t,-50,1,100);                               
        x = x+chirp(t,200,1,350); 

    elseif nb == 4                     % 3 chirps drawing a triangle
 
        N = 2048;
       fe = sqrt(N);
     % axe des temps centre en zero  
       t = (-N/2:N/2-1)*(1/fe);
     % premnier chirp
       indeb1 = 20;
       infin1 = 1200;
       off1 = 30;
       p1 = 0.4;
       phi1 = p1*pi/2;
       x1 = [zeros(1, indeb1 -1) (exp(i*pi*(off1*t(indeb1:infin1)+tan(phi1)*t(indeb1:infin1).^2 ))) zeros(1,N-infin1)];
     % Second chirp
       indeb2 = 1;
       infin2 = 2048;
       off2 = 7;
       p2 = 0.05;
       phi2 = p2*pi/2;
       x2 = [zeros(1, indeb2 -1) (exp(i*pi*(off2*t(indeb2:infin2)+tan(phi2)*t(indeb2:infin2).^2 ))) zeros(1,N-infin2)];
     % Troisieme chirp
       indeb3 = 600;
       infin3 = 2048;
       off3 = 15;
       p3 = -0.65;
       phi3 = p3*pi/2;
       x3 = [zeros(1, indeb3 -1) (exp(i*pi*(off3*t(indeb3:infin3)+tan(phi3)*t(indeb3:infin3).^2 ))) zeros(1,N-infin3)];
     % Ajout d'un bruit
       m = 0;
       var = 0.5;
       b = m + var * randn(1,N);
     % Signal
       x = x1 +x2 + x3 + b;

 
    elseif nb == 5
        x = sig;
    end
 
    FrtDemo('slide')
 
elseif strcmp(action,'info'),
    set(gcf,'pointer','arrow')
    ttlStr = get(gcf,'Name');
	hlpStr1 = [ ...
                 ' This demo is ment to present you the Fractional      '
                 ' Fourier Transform and also an application            '
                 ' of filtering in fractional domains                   '
                 '                                                      '
                 ' How does it work ?                                   '
                 '                                                      '
                 ' The slide bar changes the value of the angle         '
                 ' at which the Fractional Transform is performed       '
                 '                                                      '
                 ' The current alpha value is written in the title of   '
                 ' the upper plot                                       '
                 '                                                      '
                 ' The POPUP menu allows you to choose, as a tutorial   '
                 ' between differents signals :                         '
                 '   - The Rectangle function that you can see evolve   '
                 '     from its temporal form to its well known         '
                 '     frequential form : the cardinal sine             '
                 '   - A Linear Chirp : just try to find in which       '
                 '     fractional domain it becomes a Dirac             '
                 '                                                      '
                 ' The filtering trick is explained in the next page    '
		 '                                                      '];
		 
	hlpStr2 = [ ...
		 ' This demo allows you to filter in Fractional Domains '  
                 ' Here''s how to proceed :                              '
                 '                                                      '
                 ' First select with the slide bar the domain you want  '
                 ' to filter in                                         '
                 '                                                      '
                 ' Then push the Filter Button To open a new window     '
                 ' containing the modulus of the fractional transform   '
                 '                                                      '
                 ' Left Click in the drawing two points delimiting      '
                 ' the area you want to keep when filtering             '
                 '                                                      '
                 ' The outer part of the FRT will be set to zero        '
                 ' and the inverse FRT is performed on the filtered     '
                 '                                                      '
                 ' The synthesized signal is obtained by performing     '
                 ' the inverse FRT.                                     '
                 '                                                      '
                 ' Both original and filtered signals are displayed     '
                 '                                                      '
                 ' As a tutorial you can select "4chirps" in the signal '
                 ' pop-up menu, then fix the slidebar at 0.814 and try  '
                 ' to isolate the chirps with filter                    '
                 ' You can also try to extract chirps from noise in     '
                 ' the signal "Chirp triangle" : the slope of the chirps'
                 ' to extract are : 0.4 0.05 -0.65                      '
                 ' Add 1 to these slopes to extract the chirps          '
                 '                                                      '];

	hlpStr3 = [ ...
                ' If you want to test it on your signals just put it as  '
                ' an input when calling FrtDemo.                         '
                '                                                        '
                ' The spectrogram checkbox toggles                       '
                ' the time display or spectrogram display when           '
                ' filtering                                              '
                '                                                        '
                ' Gilles Gonon - 13/09/2000                              '                                
                '                                                        '];

    myFig = gcf;
    helpfun(ttlStr,hlpStr1,hlpStr2,hlpStr3);
    return  % avoid fancy, self-modifying code which
    % is killing the callback to this window's close button
    % if you press the info button more than once.
    % Also, a bug on Windows MATLAB is killing the 
    % callback if you hit the info button even once!


end    % if strcmp(action, ...



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% 	Other Functions


% Fractional Fourier Transform

function frt_x=fracft(x,a)
% Gilles Gonon
N=length(x); 

phi=a*pi/2;  

fe = sqrt(N);
t = (-N/2:N/2-1)*1/fe; 	% temps normal: N

x=x(:).';

e_1 = exp(-j*(pi)*tan(phi/2)*t.^2);
x_int1 = x.*e_1;
tfx_int1 = fft(x_int1);
e_2 = exp(-j*(pi)*sin(phi)*t.^2);
e_dec = [ e_2(N/2+1:N) e_2(1:N/2)];
tfx_int2 = tfx_int1 .* e_dec;
x_int2 = ifft(tfx_int2);

x_a = x_int2.*e_1;
fact = exp(-j*pi/4*sign(sin(phi))-j*phi/2-pi/4);
frt_x = x_a*fact;
