function frt_x=frt(x,a)

%--------------------------------------------------------
%	 TF fractionnaire (FRT) de la sequence x
%
% Parametres d'appel :
% 	frt_x=frt(x,a)
%
%    x 		signal a analyser
%    a	Ordre de la FRT (entre  0 et 1)
%		0 -> 0    1 -> pi/2
%--------------------------------------------------------

% Gilles Gonon
% Modification le 16-05-2000 (Repliement).

% Test des parametres d'entree
if nargin ~= 2 
	disp('mauvais nombre d entree')
end

% On complte x pour que sa longueur soit une puissance de deux
% et de manire  supprimer le repliement 
% (projection -> cercle -> sqrt(2) points )
x = x(:);
N = length(x);
x = [ x;zeros(ceil(N*sqrt(2))-N,1) ];
x = PadDyad(x);

N=length(x);  %taille de la sequence temporelle

% Angle de la frt
phi=a*pi/2;  

fe = sqrt(N);

% Axes de temps
t = (-N/2:N/2-1)*1/fe; 	% temps normal: N

% on s'assure que x est un vecteur ligne
 x=x(:).';


%-------------------------------------------------------
%			 1 ere etape
% Multiplication par un chirp (facteur de phase)
%-------------------------------------------------------

%-------- definition du facteur de phase-----------
e_1 = exp(-j*(pi)*tan(phi/2)*t.^2);

x_int1 = x.*e_1;

%-------------------------------------------------------
%			 2 eme etape
% Multiplication par un facteur de phase dans Fourier
%-------------------------------------------------------


tfx_int1 = fft(x_int1);

%-------- definition du facteur de phase-----------

e_2 = exp(-j*(pi)*sin(phi)*t.^2);

% Decalage du chirp par rapport a l'origine
e_dec = [ e_2(N/2+1:N) e_2(1:N/2)];
% de 	   0 a N/2-1    -N/2 a -1

tfx_int2 = tfx_int1 .* e_dec;

x_int2 = ifft(tfx_int2);

%-------------------------------------------------------
%			 3 eme etape
%	      Nouvelle multiplication par e_1
%-------------------------------------------------------

x_a = x_int2.*e_1;


%-------------------------------------------------------
%		  Normalisation du resultat
%-------------------------------------------------------

fact = exp(-j*pi/4*sign(sin(phi))-j*phi/2-pi/4);
frt_x = x_a*fact;